from random import sample
from time import time

def gnomesort(A):
    '''Sortiert Array A inplace/aufsteigend mit Gnomesort.'''
    i = 0
    while i < len(A):
        if i == 0:
            i = i + 1
        elif A[i-1] > A[i]:
            A[i-1], A[i] = A[i], A[i-1]
            i = i - 1
        else:
            i = i + 1

############################################################

def selectionsort(A):
    '''Sortiert Array A inplace/aufsteigend mit Selectionsort'''
    n = len(A)
    for i in range(0, n-1):
        minpos = i # Positionskandidat
        for j in range(i+1, n):
            if A[j] < A[minpos]: 
                minpos = j
        A[minpos], A[i] = A[i], A[minpos]

############################################################

def insertionsort(A):
    '''Sortiert Array A aufsteigend/inplace mit Insertionsort''' 
    for i in range(1, len(A)):
        x = A[i] # das einzusortierende Element
        j = i-1  # aktuelle Position von 'cand'
        while(j >= 0 and A[j] > x):
            A[j+1] = A[j]
            j = j-1
        A[j+1] = x # Fülle die Lücke

        
############################################################

def mergesort(A):
    mergesort_helper(A, 0, len(A))

def mergesort_helper(A, left, right):
    if (right - left) > 1:
        mid = (left + right) // 2
        mergesort_helper(A, left, mid)
        mergesort_helper(A, mid, right)
        merge(A, left, mid, right)

def merge(A, left, mid, right):
    L = A[left:mid] + [float('inf')]
    R = A[mid:right] + [float('inf')]
    i = 0
    j = 0
    for k in range(left, right):
        if L[i] <= R[j]:
            A[k] = L[i]
            i = i+1
        else:
            A[k] = R[j]
            j = j+1
            

############################################################

def randlist(n):
    return sample(range(0,n), n)

reps = 10

fd = open('sort_benchmark.csv', mode='w')
fd.write('n\tgnome\tselection\tinsert\tmerge\n')        

for n in range(1000, 9000, 1000):
    tg = ts = ti = tm = 0        
    print(f'Sortiere {reps} Listen der Länge {n} ...')
    for i in range(0, reps):
        L = randlist(n)
        #L = [i for i in range(0, n)] # bereits sortiert
        A = L[:]
        tg1 = time()        
        gnomesort(A)
        tg2 = time()
        A = L[:]
        ts1 = time()        
        selectionsort(A)
        ts2 = time()
        A = L[:]
        ti1 = time()        
        insertionsort(A)
        ti2 = time()
        A = L[:]
        tm1 = time()
        mergesort(A)
        tm2 = time()
        
    tg += (tg2-tg1)
    ts += (ts2-ts1)
    ti += (ti2-ti1)
    tm += (tm2-tm1)

    fd.write(f'{n}\t{tg:.5f}\t{ts:.5f}\t{ti:.5f}\t{tm:.5f}\n')        
          
                
fd.close()
    

    
